// GEngine.cpp: implementation of the GEngine class.
//
//////////////////////////////////////////////////////////////////////

#include "ph.h"
#include "GEngine.h"
#include "H3DModel.h"
#include "GObject.h"
#include "GAI.h"
#include "GAI_Player1.h"
#include "GAI_Player2.h"
#include "GAI_Random.h"
#include "GSpaceShip.h"
#include "GAI_Random.h"
#include "GView.h"
#include "GBackGround.h"
#include "GActiveObject.h"
#include "GRun.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GEngine::GEngine(GRun *run)
{
	int i;

	for (i = 0; i < MAX_OBJECTS; i++) {
		m_object[i] = NULL;
		m_object_del[i] = NULL;
	}
	m_objects = 0;
	m_objects_del = 0; // pocetr objektu na smazani

	m_run = run; // prirazeni GRun

	// tvorenmi mista pro modely
	for (i = 0; i < MAX_3D_MODELS; i++)
		m_3Dmodel[i] = new H3DModel;

	// nacitani modelu
	m_3Dmodel[0]->LoadModel("ship.3ds");
	m_3Dmodel[1]->LoadModel("ship2.3ds");
   m_3Dmodel[3]->LoadModel("projectil.txt");
	m_3Dmodel[4]->LoadModel("lives_right.txt");
	m_3Dmodel[5]->LoadModel("lives_left.txt");
	m_3Dmodel[6]->LoadModel("zat.3ds");
	m_3Dmodel[7]->LoadModel("zat2.3ds");
	
		
	m_view.Init(this);

	m_paused = false;
}

/////////////////////////////////////////////////////////////////////////////////////////

GEngine::~GEngine()
{
	int i;

   // ruseni objektu
	while (m_objects > 0)
		m_object[0]->SelfDelete();

	// ruseni modelu
	for (i = 0; i < MAX_3D_MODELS; i++)
		if (m_3Dmodel[i]) delete m_3Dmodel[i];

}

/////////////////////////////////////////////////////////////////////////////////////////

void GEngine::Render()
{
	 m_view.Render();
}

/////////////////////////////////////////////////////////////////////////////////////////

void GEngine::Move()
{
	// natoceni raketek, aby byly stale proti sobe
	if (((GActiveObject*)m_object[0])->m_position.x > ((GActiveObject*)m_object[1])->m_position.x) {
		((GActiveObject*)m_object[0])->m_rotation.y = 0;
		((GActiveObject*)m_object[1])->m_rotation.y = 180;
	}
	else {
		((GActiveObject*)m_object[1])->m_rotation.y = 0;
		((GActiveObject*)m_object[0])->m_rotation.y = 180;
	}

	// otestujem, zda nenastaly podminky pro konec hry
	TestGameEnd();


/*	
/// Ovladani enginu hry
	Uint8 *keystate = SDL_GetKeyState(NULL);

	// posouva kamrou
	if (keystate[SDLK_KP_PLUS]) m_view.m_camera_Z -= 1;
	if (keystate[SDLK_KP_MINUS]) m_view.m_camera_Z += 1;
*/	
}

/////////////////////////////////////////////////////////////////////////////////////////

void GEngine::TimePassed(int milisec)
{
	// zda nahodou neni pauza
	if (m_paused) return;

	m_TimeManager.DoEvents(milisec); // pohne objekty, ktere jsou na rade

	Move(); // aktualizuje stav hry

	DeleteObjects(); // smazeme objekty, ktere si o to rekly


// BERNY: to je tady, aby se zjistovalo, jak rychle se volame - kolik ms trva 1 smycka
//	fprintf(stderr,"%dms\n",milisec);
}

/////////////////////////////////////////////////////////////////////////////////////////

void GEngine::StartGame(int control_L, int control_R)
{
	// hrac 1
	GSpaceShip *pl1 = new GSpaceShip(this, m_3Dmodel[0], m_3Dmodel[6]);

	// nastaveni ovladani
	GAI *ai = NULL;
	switch(control_R) {
	case 0 : ai = new GAI_Player1(this, pl1); break;
	case 1 : ai = new GAI_Player2(this, pl1); break;
	case 2 : ai = new GAI_Random(this, pl1); break;
	}
 	pl1->SetAI(ai);

  // nastaveni pozice
	pl1->m_position.x =  20;
	pl1->m_position.y = -18;
	pl1->m_position.z = -50;

	// pridani
	AddObject(pl1);

	// hrac 2
	GSpaceShip *pl2 = new GSpaceShip(this, m_3Dmodel[1], m_3Dmodel[7]);

	// nastaveni ovladani
	ai = NULL;
	switch(control_L) {
	case 0 : ai = new GAI_Player1(this, pl2); break;
	case 1 : ai = new GAI_Player2(this, pl2); break;
	case 2 : ai = new GAI_Random(this, pl2); break;
//	default ai = NULL;
	}
	pl2->SetAI(ai);

	// nastaveni pozice
	pl2->m_position.x = -20;
	pl2->m_position.y =  18;
	pl2->m_position.z = -50;

   // pridani
	AddObject(pl2);

	// pozadi
	GBackGround *back = new GBackGround(this);
	AddObject(back);

	m_objects_del = 0; // pocetr objektu na smazani

	m_paused = false; // preci nebudeme zacinat hru zapauzovanou :-)
}

/////////////////////////////////////////////////////////////////////////////////////////

bool GEngine::AddObject(GObject *object)
{
	if (m_objects >= MAX_OBJECTS || object == NULL) return false;

	m_object[m_objects++] = object;

   m_TimeManager.AddTimeEvent(object);
   object->m_bValid     = TRUE;

	return true;
}

/////////////////////////////////////////////////////////////////////////////////////////

void GEngine::RemoveObject(GObject *object)
{
	// odebrani obejktu z casoveho rizeni udalosti
	m_TimeManager.RemoveTimeEvent(object);

	// odebrani z enginu
	for (int i = 0; i < m_objects; i++)
		if (m_object[i] == object) {

			m_object[i] = m_object[m_objects-1];

			m_object[m_objects-1] = NULL;

			m_objects--;

			return;
		}
}

/////////////////////////////////////////////////////////////////////////////////////////

void GEngine::DeleteObjects()
{
	// vsechny objekty, ktere si daly pozadavek projdeme a dovolime jim odmazani
	for (int i = 0; i < m_objects_del; i++) 
		if (m_object_del[i] != NULL) {
			m_object_del[i]->SelfDelete();
			m_object_del[i] = NULL;
		}

	m_objects_del = 0; // nastavime pocet objektu na smazani na 0
}

/////////////////////////////////////////////////////////////////////////////////////////

void GEngine::WantBeDeleted(GObject *object_to_delete)
{
	m_object_del[m_objects_del++] = object_to_delete; // pouhe zaregistrovani pozadavku o smazani
}

/////////////////////////////////////////////////////////////////////////////////////////

void GEngine::KeyDown(Uint8 key)
{
	switch (key) {
	case SDLK_p :      // zapauzujem hru
		m_paused = !m_paused;
		break;
/*	case SDLK_j :      // zapneme 2. hrace s IQ pocitace (random)
		((GActiveObject*)m_object[1])->SetAI(new GAI_Random(this, (GActiveObject*)m_object[1]));
		break;
	case SDLK_k :      // zapneme 2. hrace s IQ pocitace (random)
		((GActiveObject*)m_object[0])->SetAI(new GAI_Random(this, (GActiveObject*)m_object[0]));
		break; */
	case SDLK_ESCAPE:
		m_run->StartMenu();
		break;
		
	}
}

/////////////////////////////////////////////////////////////////////////////////////////

inline float abs(float f)
{
	if (f < 0) return -f;
	return f;
};

void GEngine::HitTest(GActiveObject *ob1)
{
	GActiveObject *ob2 = NULL;

	// projdeme vsechny objekty
	for (int i = 0; i < m_objects; i++)
		// a vybereme jen GActive Objekty
      if (IS_AOBJECT(m_object[i]) && ob1 != m_object[i]) {
			ob2 = (GActiveObject*) m_object[i];

			// testujeme na naraz - vsechny souradnice musi byt bliz, nez "dovoleno"
			if (abs(ob1->m_position.x - ob2->m_position.x) < abs(ob1->m_size.x + ob2->m_size.x) &&
			    abs(ob1->m_position.y - ob2->m_position.y) < abs(ob1->m_size.y + ob2->m_size.y) &&
			    abs(ob1->m_position.z - ob2->m_position.z) < abs(ob1->m_size.z + ob2->m_size.z)) {
				// pri narazu zkusime, zda bude treba upravit pozici objektu a jejich smery
				// chceme, aby se obe strany dozvedely o stretu - nelze pouzit if ( && ) kuli optimalizacim kompilatoru - svine jedna :-)
            bool undomove = true; // zda je potreba undomove
				undomove = undomove && ob1->Hitted(ob2); // oznameni o stretu
 				undomove = undomove && ob2->Hitted(ob1);
     			if (undomove) {
			  		ob1->m_position = ob1->m_old.m_position; // odsuneme zpet
			    	// odtrceni - prohozeni rychlosti - dokonale pruzna srazka :-)
			      CVector3 ob1move = ob1->m_moveing;
			      // chtelo by to pretizit operatory :-)
			      ob1->m_moveing.x = (ob2->m_moveing.x*ob2->m_veight)/ob1->m_veight;
			      ob1->m_moveing.y = (ob2->m_moveing.y*ob2->m_veight)/ob1->m_veight;
			      ob1->m_moveing.z = (ob2->m_moveing.z*ob2->m_veight)/ob1->m_veight;
   		      ob2->m_moveing.x = (ob1move.x*ob1->m_veight)/ob2->m_veight;
   		      ob2->m_moveing.y = (ob1move.y*ob1->m_veight)/ob2->m_veight;
   		      ob2->m_moveing.z = (ob1move.z*ob1->m_veight)/ob2->m_veight;
			  	} // if (undomove
			}  // if (abs(...
		} // if (IS_ACTIVE...
	
 /*
	GEngine::HitTest(..);
Projde vsechny objekty (pripadne se objekty utridi a bude se hledat v "okruhu") a pro kazdy
otestuje, zda nenastala kolize.
Neni-li objekt odvozen od GActiveObject, plati (m_type & 1) == 0 a nemuze se ucastnit kolize.
pokud ano, oznamime objektum kolizi
  obj1->Hitted(obj2) a obj1->Hitted(obj2)
funkce Hitted(GObject*) vrati true, pokud bylo teleso "tuhe" a je potreba obstrcit. Poklud
vrati false znamena to, ze kolize se vyresila bez nutnosti menit souradnice (prunikem
objektu nebo znicenim (ne nutne obou) )
if (obj1->Hitted(obj2) && obj1->Hitted(obj2)) {
objekty se od sebe posunou tak, aby nebyly v koliznim prostoru - obj1 se vrati
na puvodni souradnice a podle zakona p = m*v se prohodi vzajemne GActive::m_moveing
}

Je-li hitted volano pro strelu a strela zjisti, ze druhym objektem je lod, ktere chce uskodit,
zavola GShip::LostLives(int lives)
*/
	
}

// zkousi, zda je konec hry (dosli zivoty....)
void GEngine::TestGameEnd()
{
	// testuje, zda dosly zivoty
	if (((GSpaceShip*)m_object[0])->m_lives <= 0 ||
	    ((GSpaceShip*)m_object[1])->m_lives <= 0) {

	    // vyplneni struktoru game status
       st_game_status gs;
       gs.ended = true;
       gs.lives_L = ((GSpaceShip*)m_object[1])->m_lives;
       gs.lives_R = ((GSpaceShip*)m_object[0])->m_lives;

       // odeslani GRunu - prepne se do vysledkove listiny
       m_run->GameEnded(&gs);
       return; // za tim volanim uz se nesmi volat nic jineho!! - return, aby se na to nezapomnelo
   }
}
