// GRun.cpp: implementation of the GRun class.
//
//////////////////////////////////////////////////////////////////////

#include "ph.h"
#include "GRun.h"

#include "MMenu.h"
#include "GEngine.h"
#include "mvictory.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GRun::GRun()
{
	m_game_speed = 1;
	m_actual = NULL;
	m_running = true;
	m_loading = false;
}

GRun::~GRun()
{
	// odmazani neodmazaneho
	SAFE_DELETE(m_actual);
}

void GRun::Render()
{
	// smazani stare sceny
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// kresleni aktualniho
	if (m_actual != NULL) m_actual->Render();

	// sem se pripadne muze jeste neco pridat (treba oznaceni betaverze...)

	// prohodi buffery - zobrazi to, co se vyrenderovalo
	SDL_GL_SwapBuffers();
}

void GRun::TimePassed(int milisec)
{
	// zda se nahodou nenacitalo (menilo m_actual) v tom pripade vynechame jednu smycku
	if (m_loading) {
   	m_loading = false;
    	return;
	}
	// upraveni casu podle rychlosti hry a predani m_actual
	if (m_actual != NULL) {
		milisec = (int) (m_game_speed * (double)milisec);
		m_actual->TimePassed(milisec);
	}
}

// obsluha stisku klavesy
void GRun::KeyDown(Uint8 key)
{
	if (m_actual != NULL) m_actual->KeyDown(key);
}

// obsluha uvolneni klavesy
void GRun::KeyUp(Uint8 key)
{
	if (m_actual != NULL) m_actual->KeyUp(key);
}

// Ukonci program
void GRun::ExitProgram()
{
	// pozzadavek na ukonceni programu - behovou promenou nastavi na false
	m_running = false;
}

// odstratuje hru - jako parametry se predaji kdo co ridi
void GRun::StartGame(int control_L, int control_R)
{
	// odstraneni stareho
	SAFE_DELETE(m_actual);

	// vytvoreni noveho enginu
   GEngine *engine = new GEngine(this);

	// predame parametry, jaka hra ma byt (kdo co ovlada)
	engine->StartGame(control_L, control_R);

	// aktivujeme engine - nastavime, aby semu posilaly message
	m_actual = engine;

	// nastavime m_loading
	m_loading = true;	
}

// hodi hru do menu
void GRun::StartMenu()
{
	// odstraneni stareho
	SAFE_DELETE(m_actual)

	// vytvoreni noveho menu
   m_actual = new MMenu(this);

	// nastavime m_loading
	m_loading = true;
}

// GEngine oznamuje GRunu, ze hra byla dohrana a s jakym vysledkem (alespon jeden byl zabit)
void GRun::GameEnded(st_game_status *game_status)
{
	MVictory *vicotry = new MVictory(this, game_status);

	m_actual = vicotry;
	
	// nastavime m_loading
	m_loading = true;
}
