#include "ph.h"
#include "TimeManager.h"


CTimeEvent::CTimeEvent()
{
	m_start_time = 0;
	m_period = 1;
	m_time_manager_list_item = NULL;
	m_time_manager_index = -1;
	m_bVariablePeriod = FALSE;
	m_bFixed32Period = FALSE;
	m_bAsync = FALSE;
	m_bValid = FALSE;
	m_dwA = 1;
	m_dwB = 1;
}
      
CTimeEvent::~CTimeEvent(void)
{
   if (m_time_manager_list_item!=NULL){
   	m_time_manager_list_item->UnBind();
      delete m_time_manager_list_item;
   }
}

// nastavi periodu udalosti
// (fix32 == TRUE): nastavi periodu jako FixedPoint32 cislo A.B
// (fix32 == FALSE):
//   (B == {0,1}): nastavi normalni periodu A
//   (B != {0,1}): nastavi periodu v podilovem tvaru A/B
void CTimeEvent::SetPeriod(DWORD A, DWORD B, BOOL fix32)
{
	// BER: tady jsi mel asi chybku = misot ==
//   if (m_bFixed32Period = fix32){
   if (m_bFixed32Period == fix32) {
		if (A || B) {
			m_bVariablePeriod = TRUE;
			m_dwAccumulator = 0;
			m_dwperiod = (m_bMajorA = ((m_dwA = A) != 0))?A:1;
			m_dwB = B;
		}
		return;
   }

   if (!A) return;
   if (B < 2) {
		m_bVariablePeriod = FALSE;
		m_dwA = m_dwperiod = A;
      m_dwB = 1;
	}
	else {
		m_dwA = A; m_dwB = B;
		if (A <= B) {
			m_bMajorA = FALSE;
			m_dwperiod = 1;
			m_dwAccumulator = A;
		}else{
			m_bMajorA = TRUE;
			if (B != 65536) {
				m_dwperiod = A/B;
				m_dwAccumulator=A%B;
			}else{
				m_dwperiod = A >> 16;
				m_dwAccumulator = A & 0xFFFFL;
			}
			m_dwRemainder = m_dwAccumulator << 1;
		}
		m_bVariablePeriod = TRUE;
   }
}


CTimeManager::CTimeManager(void)
{
	m_start_time = 1;
	m_bWorking = false;
	m_bIteration = true;
	m_tmp_item = NULL;
}


CTimeManager::~CTimeManager(void)
{
	CTimeEventListItem *item, *temp;
   CTimeEvent *event;
   for (int i = 0; i < TA_MAX; i++) {
		if ((item = m_time_array[i].next) != NULL) {
			do {
				if ((event = (temp=item)->data))
					event->m_time_manager_list_item = NULL;
				item=item->next;
				delete temp;
			} while (item);
			m_time_array[i].next=NULL;
		}
	}

   item = m_async_time_list.next;

	while (item) {
		if ((event = (temp=item)->data))
			event->m_time_manager_list_item = NULL;
      item=item->next;
      delete temp;
	}

   m_async_time_list.next=NULL;
}
    
BOOL CTimeManager::AddTimeEvent(CTimeEvent *event)
{
	CTimeEventListItem *item;
   
	if (!event->m_bAsync) {
		long time0 = event->m_start_time;     // +0, +1, +2 ...

		if (time0 >= TA_LAST-1) {            
			event->m_start_time += m_start_time - TA_LAST;
			time0 = 0;
		} else {
			if (time0 < 0) return FALSE;       // chybne zadany CTimeEvent::m_start_time
			if ((time0 += m_start_time) > TA_LAST) // presahuje buffer, ale ne periodu bufferu
				time0 -= TA_LAST;
			event->m_start_time = 0;
		}
      item = m_time_array + (event->m_time_manager_index = time0);
	} 
	else item = &m_async_time_list;

	if ((item = new CTimeEventListItem(item, item->next, event)) == NULL) return FALSE;
	
	event->m_time_manager_list_item = item;

	return TRUE;
}

BOOL CTimeManager::RemoveTimeEvent(CTimeEvent *event)
{
	if (!event) return FALSE; // NULL neni platny event

	CTimeEventListItem* &item = event->m_time_manager_list_item;
	
	if (!item)  return FALSE; // event neni v TimeManageru
  
   if ((m_bWorking) && (item == m_tmp_item)){
		// odstranujeme event, se kterym se prave pracuje
		m_bIteration = FALSE;        // iteraci provedeme jiz zde
		m_tmp_item = m_tmp_item->next; // posun na dalsi item
	}
	event->m_time_manager_index = -1;
	event->m_bValid = FALSE;       // odstraneny event bude neplatny
	item->UnBind();            // unbindne item z TimeManageru
	delete item;               // smaze item (event zachovava)
	item = NULL;                 // zrusi odkaz v eventu do TimeManageru

	return TRUE;
}

// zmeni periodu, o pozici v bufferu se stara, pracuje-li se s eventem
void CTimeManager::SetPeriod(CTimeEvent *event, DWORD A, DWORD B)
{
	if (!event) return;
	CTimeEventListItem *&item = event->m_time_manager_list_item;

	event->SetPeriod(A, B, event->m_bFixed32Period);
  
	if ((!m_bWorking) || (!item) || (item != m_tmp_item)) return;
  
  //PostMessage(g_hwndMain, WM_CLOSE, 0, 0);

  // item ceka v TimeManageru - s eventem prave pracujeme
	m_bIteration = FALSE;
	m_tmp_item = m_tmp_item->next;  

	if (!event->m_bAsync){
  // synchronni udalost (asynchronni se nemuseji presunovat (neni kam))
		long time0 = event->m_period;
		if (time0 >= TA_LAST-1){
			event->m_start_time = time0 - TA_MAX + m_start_time - 1;
			time0 = 0;
		}else{
			if (time0 < 0) time0 = 0;
			if ((time0 += m_start_time-1) > TA_LAST)
			time0 -= TA_LAST;
		}
		event->m_time_manager_index = time0;
		item->UnBind();
		item->Bind(m_time_array+time0, m_time_array[time0].next);
	}
}

//-------------------------------------------------
// void CTimeManager::SetPosition(CTimeEvent, BOOL)   {NETESTOVANO}
//-------------------------------------------------
// nastavi novou pozici eventu v bufferu podle event->m_start_time
// pokud ActualStartTime==TRUE, pak bude nova pozice:
//   CTimeManager::m_start_time+event->m_start_time
// pokud ActualStartTime==FALSE, pak bude nova pozice:
//   event->m_time_manager_index+event->m_start_time
// provede se bezpecne zarazeni do bufferu (je osetreno preteceni
// bufferu i moznost, ze presunujeme event, se kterym se prave pracuje)

void CTimeManager::SetPosition(CTimeEvent *event, BOOL ActualStartTime)
{
	if ((!event) || (event->m_bAsync) || (event->m_start_time < 0)) return;
   CTimeEventListItem *item = event->m_time_manager_list_item;

   long time0;
  
   if (!ActualStartTime) {
		// konverze m_time_manager_index do m_start_time
		if ((time0 = event->m_time_manager_index) < m_start_time)
			time0 += TA_LAST - m_start_time;
		else
			time0 -= m_start_time;
		time0 = (event->m_start_time += time0);
   } 
	else time0 = event->m_start_time;
  
   if (time0 >= TA_LAST-1) {
		event->m_start_time += m_start_time - TA_LAST;
		time0 = 0;
	}else{
		if ((time0 += m_start_time) > TA_LAST)
			time0 -= TA_LAST;
		event->m_start_time = 0;
	}

	event->m_time_manager_index = time0;
  
  // pokud s eventem prave pracujeme, preskocime ho
   if ((m_bWorking) && (item == m_tmp_item)){
		m_tmp_item = m_tmp_item->next;
		m_bIteration = FALSE;
   }

   item->UnBind();
   item->Bind(m_time_array+time0, m_time_array[time0].next);
}

// CTimeManager::m_start_time zahrnuje index, ktery se ma zpracovat
void CTimeManager::DoEvents(long time_interval)
{
   CTimeEventListItem *temp, *entry, *&item = m_tmp_item;
   CTimeEvent         *event;
   long                end, time0;

   union{long ltemp; DWORD dwtemp;};
  
   m_bWorking = true; // nastavime priznak, ze TimeManager prave obsluhuje eventy
  
  // 1. OBSLUHA ASYNCHRONNICH UDALOSTI ----------------
  // asynchr. udalosti jsou ulozeny v seznamu m_async_time_list
   item = m_async_time_list.next;            // prvni asynchr. udalost
   while (item) {                         
		if ( (event=item->data)->m_bValid ){
			event->DoEvent(time_interval);    // provede event asynchr. udalosti
			if (!m_bIteration) { m_bIteration = TRUE; continue; }
		}
		item = item->next;              // dalsi asynchr. udalost v seznamu
   }
  
  // 2. OBSLUHA SYNCHRONNICH UDALOSTI -----------------
  // synchr. udalosti jsou ulozeny v seznamech v bufferu m_time_array
	while (time_interval) {                 // dokud zbyva casovy interval
		if (m_start_time + time_interval > TA_MAX){  
      // presahuje hranici bufferu
			time_interval -= TA_MAX - m_start_time; // odecte se kolik casu zbyva do konce bufferu
			end = TA_LAST;                      // posledni prvek, ktery v teto iteraci zpracovat
		}else{
			// neprsahuje hranici bufferu
			end = m_start_time + time_interval - 1;   // posledni prvek, ktery v teto iteraci zpracovat
			time_interval = 0;                  // zpracovan vsechen cas
		}
    
		// zpracuje synchr. udalosti v m_time_array[m_start_time..end]
      // m_start_time++ znamena prechod na dalsi casovou skupinu
		entry = m_time_array + m_start_time; // seznam udalosti,ktere jsou na rade
		for (++m_start_time; m_start_time <= end+1; m_start_time++, entry++) {
			item = entry->next; // prvni udalost seznamu
			while (item) {
				if (!(event=item->data)->m_bValid) { item = item->next; continue; }
				event->DoEvent();     // provede udalost
				if (!m_bIteration) { m_bIteration = TRUE; continue;}
				if (!event->m_bValid) { item = item->next; continue; }
				time0 = event->m_period;  // ulozi periodu
        
				if (event->m_bVariablePeriod) {
					// zpracuje udalosti s promenlivou periodou
					if (event->m_bFixed32Period){
						// perioda je ve tvaru FixedPoint32 [m_dwA.m_dwB]
						if (event->m_bMajorA){
						// perioda >= 1
						// pricte m_dwB k dwAcc. - po preteceni zvetsi time0 o 1
							if (event->m_dwB>=event->m_dwAccumulator){
								if (event->m_dwB>(event->m_dwAccumulator+=event->m_dwB))
									time0++;
							} else
								if ((dwtemp=event->m_dwAccumulator)>(event->m_dwAccumulator+=event->m_dwB))
									time0++;
						} else {
							// perioda < 1
							// pricita m_dwB k dwAcc. - provadi udalost az do preteceni
							dwtemp = (event->m_dwB>event->m_dwAccumulator)?event->m_dwB:event->m_dwAccumulator;
							while(dwtemp > (event->m_dwAccumulator += event->m_dwB)){
								event->DoEvent();
								if ((!m_bIteration) || (!event->m_bValid)) break;
								dwtemp = event->m_dwAccumulator;
							}
							if (!m_bIteration){ m_bIteration = TRUE; continue; }
							if (!event->m_bValid){ item = item->next; continue; }
						}
					} else {
						// perioda je v podilovem tvaru m_dwA/m_dwB
						if (event->m_bMajorA){
							// perioda je realne cislo > 1
							// podle internich promennych rozhodne, zda bude perioda
							// coby cele cislo o 1 vetsi a aktualizuje promenne
							if ((event->m_dwAccumulator += event->m_dwRemainder) >= event->m_dwB<<1){
								event->m_dwAccumulator-=event->m_dwB<<1;
								time0++;
							}
						}else{
							// perioda je realne cislo z intervalu (0,1>
							// podle internich promennych provadi eventy udalosti kolikrat
							// je potreba. perioda je pak reprezentovana jen jako cislo 1
							ltemp = long(event->m_dwA<<1);
							event->m_lAccumulator -= long(event->m_dwB<<1);
							while((event->m_lAccumulator += ltemp) < 0){
								event->DoEvent();
								if ((!m_bIteration) || (!event->m_bValid)) break;
							}
							if (!m_bIteration){ m_bIteration = TRUE; continue; }
							if (!event->m_bValid){ item=item->next; continue; }
						}
					}
				}

				if (time0 >= TA_LAST) {
					// perioda presahuje velikost bufferu
					// event se ulozi k pozdejsimu zpracovani (uchovava se jako
					// polozka seznamu 0. prvku bufferu m_time_array)
					event->m_start_time = time0 - TA_MAX + m_start_time - 1;
					time0 = 0; // index do m_time_array
				} else {
					// perioda nepresahuje velikost bufferu
					// konverze z time0 coby periody na index do m_time_array
					if ((time0 += m_start_time-1) > TA_LAST)
						// za hranici bufferu (pocita se zase od zacatku)
						time0 -= TA_LAST;
				}
				event->m_time_manager_index = time0;
				// posun na dalsi prvek seznamu tohoto indexu v m_time_array a
				// presun obslouzene udalosti na jeji nove misto v bufferu
		      item = (temp=item)->next;
				temp->UnBind();
				temp->Bind(m_time_array+time0, (m_time_array+time0)->next);
			}
   }

   m_start_time--;// znovu na index prvku, ktery je na rade (for - presah)

   if (end == TA_LAST){
      // bylo dosazeno preteceni bufferu
      m_start_time = 1;          // m_start_time - od zacatku bufferu (index c. 1)
      item = m_time_array->next; // item - prvni cekajici udalost
      while (item) {
         if (!(event=item->data)->m_bValid){ item = item->next; continue; }
         // time0 + m_start_time = novy index udalosti do m_time_array
         time0 = event->m_start_time; // +0, +1, +2 ...
			if (time0 >= TA_LAST){
				// index do m_time_array presahuje velikost bufferu
				// udalost se necha cekat dal s novym indexem do m_time_array
				event->m_start_time-=TA_LAST;
				item = item->next; // pokracuje se dalsim prvkem seznamu
				continue;
			}else{
          // index do m_time_array nepresahuje velikost bufferu
          // time0 mensi nez 0 je neplatny - udalost se nezpracovava
				if (time0 < 0){ item = item->next; continue; }
          // konverze time0 na index do m_time_array (pricita m_start_time==1)
				if ((++time0) > TA_LAST)
					// index presahuje hranici bufferu - pocita se od zacatku
					time0 -= TA_LAST;
					event->m_start_time = 0; // startovni cas muzeme nulovat
				}
				event->m_time_manager_index = time0;
				// posun na dalsi prvek seznamu indexu c. 0 v m_time_array a
				// presun cekajici udalosti na jeji nove misto
				item = (temp=item)->next;
				temp->UnBind();
			   temp->Bind(m_time_array+time0, m_time_array[time0].next);
			}
		}
	}
   m_bWorking = false; // TimeManager prave obslouzil eventy (zrusime priznak)
}
