#ifndef __TIMEMANAGER_INCLUDED__
#define __TIMEMANAGER_INCLUDED__

#include "ListItem.h"

#define TA_LAST 1000       // 1000ms
#define TA_MAX  (TA_LAST+1)

typedef ListItem<class CTimeEvent*> CTimeEventListItem;

/* CTimeEvent::start_time nas zajima pouze pokud je prvek v 
   time_array[0], nebo pri vkladani prvku, ci zmene jeho pozice.
   Pri osetrovani eventu v time_array[>0] nas nezajima. */

class CTimeEvent{
  friend class CTimeManager;
  public:
    CTimeEvent(void):m_start_time(0),m_period(1),m_time_manager_list_item(NULL),
      m_time_manager_index(-1),m_bVariablePeriod(FALSE),m_bFixed32Period(FALSE),
      m_bAsync(FALSE), m_bValid(FALSE), m_dwA(1), m_dwB(1){}
    virtual ~CTimeEvent(void){
      if (m_time_manager_list_item!=NULL){
        m_time_manager_list_item->UnBind();
        delete m_time_manager_list_item;
      }
    }
    BOOL isFixedPeriod(void){return m_bFixed32Period;}
    BOOL isVariablePeriod(void){return m_bVariablePeriod;}

    void SetPeriod(DWORD A, DWORD B, BOOL fix32=FALSE);

    virtual void DoEvent(void){};        // vola se, kdyz ubehne perioda
    virtual void DoEvent(long elapsed){};// pro asynchronni udalost

    long  m_start_time;        // posun v time_array pri vkladani eventu
    BOOL  m_bAsync;            // asynchronni udalost
    BOOL  m_bValid;            // platna udalost
    union{
      long  m_period;          // perioda casove udalosti (period>0)
      DWORD m_dwperiod;        // stejna promenna, ale unsigned
    };
    union{
      struct{DWORD m_dwB,m_dwA;};// def. var. periodu
      ULONGLONG m_dwA_dwB;     // slozeni do fixed32 z dwA.dwB
    };

  private:  
    BOOL  m_bFixed32Period;    // perioda ve tvaru FixedPoint32 [dwA.dwB]
    BOOL  m_bVariablePeriod;   // variabilni perioda (dwA/dwB)
    BOOL  m_bMajorA;           // TRUE:dwA>dwB x FALSE:dwA<=dwB
    union{
      DWORD m_dwAccumulator;   // pom. promenna pro vypocet periody
      long  m_lAccumulator;   // stejna promenna, ale signed
    };
    DWORD m_dwRemainder;       // zbytek po deleni dwA/dwB
    int m_time_manager_index;  // prislusnost k casove skupine TimeManageru
    CTimeEventListItem *m_time_manager_list_item; // odkaz na prislusnou polozku v manageru
};

class CTimeManager{
  public:
    CTimeManager(void):m_start_time(1),m_bWorking(false),m_bIteration(true),
                       m_tmp_item(NULL){}
   ~CTimeManager(void);

    BOOL AddTimeEvent(CTimeEvent *event);
    BOOL RemoveTimeEvent(CTimeEvent *event);

    void SetPeriod(CTimeEvent *event, DWORD A, DWORD B);
    void SetPosition(CTimeEvent *event, BOOL ActualStartTime);
    void DoEvents(long time_interval);

    bool isWorking(void){return m_bWorking;}

  private:
    CTimeEventListItem m_time_array[TA_MAX]; // prvek 0 se pouziva pro eventy, ktere se nevejdou do time_array
    CTimeEventListItem m_async_time_list;    // seznam asynchronnich udalosti
    CTimeEventListItem *m_tmp_item;          // item, ktery se prave zpracovava

    bool m_bIteration; // bud probehne iterace cyklu, nebo se provede continue;
    bool m_bWorking;   // TRUE v prubehu CTimeManager::DoEvents(long)
    long m_start_time; // index do pole time_array (=ktery index se ma priste zpracovat)
                     // pokud je start_time==1, tak pokud ubehne 1ms, zpracuje se
                     // time_array[1] a start_time se posune na start_time==2
};

#endif//__TIMEMANAGER_INCLUDED__
