// GActiveObject.cpp: implementation of the GActiveObject class.
//
//////////////////////////////////////////////////////////////////////

#include "ph.h"
#include "GActiveObject.h"
#include "GAI.h"
#include "H3DModel.h"
#include "GEngine.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GActiveObject::GActiveObject(GEngine *engine, H3DModel *model3D):
GObject(engine)
{
	m_speed = 0;
	m_acceleration = 1;
	
	m_up = false;
	m_down = false;
	m_left = false;
	m_right = false;
	m_action = false;

	m_3Dmodel = model3D;

	m_veight = 1;
	
	m_ai = NULL;


	// nulovani vektoru
	ZeroVector(&m_moveing);
	ZeroVector(&m_size);
	ZeroVector(&m_moveing);
	ZeroVector(&m_req_movement);
	ZeroVector(&m_rotation);
}

/////////////////////////////////////////////////////////////////////////////////////////

GActiveObject::~GActiveObject()
{
	if (m_ai != NULL) delete m_ai;
}

/////////////////////////////////////////////////////////////////////////////////////////

void GActiveObject::DoEvent()
{
	// dejme sanci AI
	if (m_ai != NULL) m_ai->Move();

	// ulozeni stareho
	m_old.m_position = m_position;
	
	// reset pohybu
	m_req_movement.x = 0;
	m_req_movement.y = 0;
	m_req_movement.z = 0;

	// nastaveni req_move
	if (m_up)   m_req_movement.y = m_speed;
	if (m_down) m_req_movement.y = -m_speed;
	if (m_left) m_req_movement.x = -m_speed;
	if (m_right)   m_req_movement.x = m_speed;
 
	// zmena nastaveni aktualniho smeru/rychlosti
	m_moveing.x = m_moveing.x * (1 - m_acceleration) + m_acceleration * m_req_movement.x;
	m_moveing.y = m_moveing.y * (1 - m_acceleration) + m_acceleration * m_req_movement.y;
	m_moveing.z = m_moveing.z * (1 - m_acceleration) + m_acceleration * m_req_movement.z;

	// vlastni posun souradnic
	m_position.x += m_moveing.x;
	m_position.y += m_moveing.y;
	m_position.z += m_moveing.z;

	// otestujeme na narazy
	m_engine->HitTest(this);
}

/////////////////////////////////////////////////////////////////////////////////////////

void GActiveObject::Render()
{
	// Zatek prostorov matice
	glPushMatrix();

	// nastavime pozici modelu

	// posunuti
	glTranslated(m_position.x, m_position.y, m_position.z); 
	
	// rotace 
	glRotated(m_rotation.x, 1.0, 0.0, 0.0); 
	glRotated(m_rotation.y, 0.0, 1.0, 0.0); 
	glRotated(m_rotation.z, 0.0, 0.0, 1.0); 


	// kreslime model
	m_3Dmodel->Render();
	
	// obnoveni stare matice
	glPopMatrix();
}

/////////////////////////////////////////////////////////////////////////////////////////

void GActiveObject::SetAI(GAI *ai)
{
	if (m_ai != NULL) delete m_ai; // kdyby nahodou
	m_ai = ai;
}

/////////////////////////////////////////////////////////////////////////////////////////

bool GActiveObject::Hitted(GActiveObject * /*object*/)
{
	return false; // defaultove se nedomnivame, ze je treba resit kolizi objektu
}
