#include "ph.h"
#include "GRun.h"
#include "GEngine.h"
#include "time.h"

struct st_setting
{
	int resolution_x;    // x-ovy rozmer obrazovky
	int resolution_y;    // y-ovy rozmer obrazovky
	int color_depth;     // bites per pixel
	int fullscreen;      // 0 == okno, 1 == fullscreen
	int texture_quality; // kvalita textur
	int model_quality;   // kvalita modelu              -- nepouziva se
	int back_ground;     // zda se ma pouzivat pozadi   -- nepouziva se
} g_setting = {640, 480, 0, 0, 0, 0, 1};

// zakladni objekt hry
GRun *gRun = NULL;

// Prehodi FullScreen (Zapne / Vypne)
void ToggleFullScreen()
{
	SDL_Surface *S = SDL_GetVideoSurface();												

	if(!S || (SDL_WM_ToggleFullScreen(S) != 1))							
		fprintf(stderr, "Unable to toggle fullscreen: %s\n", SDL_GetError());
}

// Nacte nastaveni TODO:
void LoadSetting()
{
	// otevreme soubor s nastavenim
	FILE *f = fopen("st3.txt", "rt");
	int   c; // pomocna promena

	// nacteme nastaveni
	if (f != NULL) {
		// nacteni dat ze souboru
		fscanf(f, "%d %d %d %d %d", &g_setting.resolution_x, &g_setting.resolution_y, &g_setting.color_depth, &g_setting.fullscreen, &g_setting.texture_quality);
		
		fclose(f);
		f = NULL;
	}	
	else {
		fprintf(stderr, "Unable to open inicialization file. Using default valueas.\n");
	}

	htexture.SetTextureGeneratingQuality(g_setting.texture_quality);
};

// Nastaveni zakladnich parametru pro OpenGL
void InitGL(int Width, int Height)	        // We call this right after our OpenGL window is created.
{
   glViewport(0, 0, Width, Height);
   glEnable(GL_TEXTURE_2D);               // Povolime textury
   glClearColor(0.0f, 0.0f, 0.0f, 0.0f);	// Nastavime barvu pozadi na cernou
   glClearDepth(1.0);				         // povolime mazani hloubkove bufferu
   glDepthFunc(GL_LESS);			      	// The Type Of Depth Test To Do
   glEnable(GL_DEPTH_TEST);					// Povolime testovani hloubky
   glShadeModel(GL_SMOOTH);					// Enables Smooth Color Shading

	glEnable(GL_LIGHT0);								// Turn on a light with defaults set
	glEnable(GL_COLOR_MATERIAL);						// Allow color

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();				// Reset The Projection Matrix

   gluPerspective(45.0f,(GLfloat)Width/(GLfloat)Height,0.1f,100.0f);	// Calculate The Aspect Ratio Of The Window

   // povolime 2D textury

   glMatrixMode(GL_MODELVIEW);
}

// Inicializace SDL
void	InitSDL()
{
	// Inicializace generatoru nahodnych cisel
	srand(time(NULL));

   // Inicializace SDL grafiky a SDL Timeru 
   if ( SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER ) < 0 ) {
      fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
      exit(1);
	}

	// nastaveni funkce pri volani exit();
	atexit(SDL_Quit);

	// chceme doublebuffering s openGL
   SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );


	// Nastavime velikost a styl okna
	unsigned int flags = SDL_OPENGL;// |SDL_RESIZABLE; // |SDL_RESIZABLE
	if (g_setting.fullscreen) flags |= SDL_FULLSCREEN;

	// vytvorime okno
   if ( SDL_SetVideoMode(g_setting.resolution_x, g_setting.resolution_y, g_setting.color_depth, flags) == NULL ) {
		fprintf(stderr, "Unable to create OpenGL screen: %s\n", SDL_GetError());
		SDL_Quit();
		exit(2);
	}
	
	// Nastaveni titulku okna
	SDL_WM_SetCaption("StarShooter III", NULL);
}

// Inicializuje objekt gRun
void InitGame()
{
	gRun = new GRun;

	if (gRun == NULL) {
		fprintf(stderr, "Unable to create game\n");
		SDL_Quit();
		exit(3);
	}

	// hodime hru do menu (kam jinam)
	gRun->StartMenu();	
}

// Zpracovava udalosti. GRun nikdy neni == NULL, ale pro jistotu stejne testujeme
bool ProcessEvents()
{
	SDL_Event event;
   while ( SDL_PollEvent(&event) ) {
		switch (event.type) {
      
		case SDL_QUIT :       // uzivatel si preje ukoncit aplikace
			return false;
			break;

		case SDL_KEYDOWN :    // stisknuta klavesa
			switch (event.key.keysym.sym) {
      	case SDLK_F1 :
       		ToggleFullScreen();
         	break;
         default :
				if (gRun != NULL) gRun->KeyDown(event.key.keysym.sym);
				break;
			}
			break;

		case SDL_KEYUP :      // uvolnena klavesa
			if (gRun != NULL) gRun->KeyUp(event.key.keysym.sym);
			break;

      case SDL_VIDEORESIZE : // zmena velikosti okna
			InitGL(event.resize.w, event.resize.h);
			break;

      case SDL_VIDEOEXPOSE : // potreba prekreslit okno
			if (gRun != NULL) gRun->Render();
			break;
      }
   }
	return true;
}

// Funkce main - co dodat  /*argc*/ jsou zakomentovane, aby prekladac nemel kecy :-)
int main(int /*argc*/, char **/*argv*/) 
{
	// Nacteni nastaveni
	LoadSetting();

	// Inicializace knihovny SDL
	InitSDL();

	// Inicializace OpenGL
	InitGL(g_setting.resolution_x, g_setting.resolution_y);
	
	// Inicializace hry
	InitGame();

	// casovani hry
	int time1 = SDL_GetTicks();
	int time2 = SDL_GetTicks();

	// Hlavni herni smycka
	while ((gRun != NULL) && (gRun->m_running)) {

		// kdyz nedopadne dobre obsluha udalosti, program se ukonci
		if (!ProcessEvents()) break;

		// pockame, az poskoci cas alespon o 1ms. jinak dalsi volani 
		// vubec nemaji smysl, protoze se scena nemohla zmenit (nemela kdy)
		while (time2 - time1 < 1) time2 = SDL_GetTicks();

		// probehne jedna smycka aplikace
		if ((SDL_GetAppState() & SDL_APPACTIVE) && (gRun != NULL)) {
			gRun->TimePassed(time2 - time1);
			gRun->Render();
		}

		time1 = time2;
   }

   // odstraneni objektu hry
   SAFE_DELETE(gRun);

   // ukonceni SDL
	SDL_Quit();

	// konec programu	
	return 1;
}
